/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        14.06.2007
 Description:    OSD Erzeugung
 
 Ich stelle diesen Sourcecode kostenlos zur Verfgung und biete daher weder 
 Support an noch garantiere ich fr seine Funktionsfhigkeit. Auerdem
 bernehme ich keine Haftung fr die Folgen seiner Nutzung.

 Der Sourcecode darf nur zu privaten Zwecken verwendet und modifiziert werden. 
 Darber hinaus gehende Verwendung bedarf meiner Zustimmung.
------------------------------------------------------------------------------*/

#include <lpc213x.h>
#include <stdlib.h>
#include <string.h>

#include "pal.h"
#include "config.h"
#include "uart.h"
#include "printf_P.h"
#include "viclowlevel.h"

//-----------------------------------------------------------------------------
void SystemInit(void)
{
	// --- enable and connect the PLL (Phase Locked Loop) ---
	// a. set multiplier and divider
	PLLCFG = MSEL | (1<<PSEL1) | (0<<PSEL0);
	// b. enable PLL
	PLLCON = (1<<PLLE);
	// c. feed sequence
	PLLFEED = PLL_FEED1;
	PLLFEED = PLL_FEED2;
	// d. wait for PLL lock (PLOCK bit is set if locked)
	while (!(PLLSTAT & (1<<PLOCK)));
	// e. connect (and enable) PLL
	PLLCON = (1<<PLLE) | (1<<PLLC);
	// f. feed sequence
	PLLFEED = PLL_FEED1;
	PLLFEED = PLL_FEED2;
	
	// --- setup and enable the MAM (Memory Accelerator Module) ---
	// a. start change by turning of the MAM (redundant)
	MAMCR = 0;	
	// b. set MAM-Fetch cycle to 3 cclk as recommended for >40MHz
	MAMTIM = MAM_FETCH;
	// c. enable MAM 
	MAMCR = MAM_MODE;
	
	// --- set VPB speed ---
	VPBDIV = VPBDIV_VAL;
	
	// --- map INT-vector ---
	#if defined(RAM_RUN)
	  MEMMAP = MEMMAP_USER_RAM_MODE;
	#elif defined(ROM_RUN)
	  MEMMAP = MEMMAP_USER_FLASH_MODE;
	#else
	#error RUN_MODE not defined!
	#endif
}

//-----------------------------------------------------------------------------
int main(void)
{  
	SystemInit();
    uart0Init(B38400, UART_8N1, UART_FIFO_8); // setup the UART 

	printf("\n\r*** START ***\n\r");	
    
    pal_timer_init();
    enableIRQ();
	
    int a;
	
	while(1)
    {
		pal_line(0,0,259,0);
		pal_line(0,0,0,335);
		pal_line(259,335,259,0);
		pal_line(259,335,0,335);
		pal_encode_str(10,10,"TV - Terminal  V1.0");
		pal_encode_str(10,11,"(c) by Ulrich Radig");
		pal_encode_str(10,12,"-------------------");
		for (a=0;a<4000000;a++){asm("nop");};
		pal_clear();
		for (a=0;a<2000000;a++){asm("nop");};
		
		for (a=0;a<335;a+=2)
		{
		pal_line(0,0,259,a);
		}
		
		for (a=0;a<4000000;a++){asm("nop");};
		pal_clear();
		for (a=0;a<2000000;a++){asm("nop");};
		pal_encode_str(10,10,"Das ist ein weiterer Test");
		pal_encode_str(10,11,"1234567890ABCD!$%&/()=?`");
		for (a=0;a<2000000;a++){asm("nop");};
    }   
return(0);
}


