/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        14.06.2007
 Description:    OSD Erzeugung
 
 
 P0.21 PWM5  Ausgang Sync  --[A]Diode[K]--1kOhm----------Video-Signal
 P0.19 MOSI  Ausgang Video --[A]Diode[K]--220Ohm __/
 P0.16 HV - Sync auf GND
------------------------------------------------------------------------------*/
#include "pal.h"

volatile unsigned char	    pal_image[PAL_BYTES_PER_CHAR_ROW * PAL_CHAR_ROWS];
volatile unsigned char 	    *pal_image_ptr;
volatile unsigned int		pal_timer_count = 0; // Timer1 Zhlerwert
volatile unsigned int		pal_line_count = 0; // Aktuelle Display-Zeile
volatile unsigned int		tmp = 0;
volatile unsigned int		pal_bytes_to_write = 0;
volatile int		        drift = 0;

//-----------------------------------------------------------------------------
// Timer1 Compare-Match Fast Interrupt Handler (ISR) 
void __attribute__ ((interrupt("IRQ"))) tc1_cmp(void) //crt0.S
{
	PAL_SYNC_LOW(); //Sync ein
	drift = T1TC;
	T1IR = 0xFF; // Timer1 Interrupt-Flags lschen
	T0TC = T1TC;
	T1MR0 = PAL_HSYNC_64_US - DRIFTOFFSET + drift; // MatchRegister0 setzen
	T0MR0 = PAL_HSYNC_64_US - 300; // MatchRegister0 setzen
	//Synchronpuls 4.8s
    if (++pal_line_count < LINE) //Ausgabe der Zeilen
    {	
		while (T1TC < PAL_HSYNC_DURATION + DRIFTOFFSET - drift); 
		PAL_SYNC_HIGH(); //Sync aus
		 
		//Horizontaloffset
		while (T1TC < (PAL_HSYNC_DURATION + PAL_HORZ_OFF + DRIFTOFFSET - drift));

		if (pal_line_count >= PAL_MIN_VERT_OFF && pal_line_count < PAL_MAX_VERT_OFF)
		{
			while(pal_bytes_to_write--)
			{
				SSPDR = tmp; // Datenbyte in den Tx-FIFO schreiben
				tmp =  (*pal_image_ptr++) << 8;
				tmp += (*pal_image_ptr++);
				while(!(SSPSR & TNF));
			}
		}
		pal_bytes_to_write = (PAL_CHAR_COLS)/2;
    }
    else
    { 
		if (pal_line_count < VORTRABANT) // 1. Halbbild VSync
		{
			while (T1TC < PAL_HALF_HSYNC_DURATION); // Synchronpuls 2.4s 
			PAL_SYNC_HIGH(); // Sync aus
		}
		else if (pal_line_count < HAUPTIMPULS) // 1. Halbbild VSync
		{
			PAL_SYNC_HIGH(); // Sync aus
			while (T1TC < PAL_HALF_HSYNC_DURATION); // Synchronpuls 2.4s 
			PAL_SYNC_LOW(); // Sync aus
		}
		else if (pal_line_count < NACHTRABANT) // 1. Halbbild VSync
		{
			while (T1TC < PAL_HALF_HSYNC_DURATION); // Synchronpuls 2.4s 
			PAL_SYNC_HIGH(); // Sync aus
		}
		else
		{
			while (T1TC < PAL_HALF_HSYNC_DURATION); // Synchronpuls 4.8s 
			PAL_SYNC_HIGH(); // Sync aus	  
			pal_line_count = 0;
			pal_image_ptr = &pal_image[0];
			tmp =  (*pal_image_ptr++) << 8;
			tmp += (*pal_image_ptr++);
			T1IR = 0xFF; // Timer1 Interrupt-Flags lschen
			VICVectAddr = 0; // Interrupt in VIC lschen
			return;
		}
		T1MR0 -= PAL_HSYNC_32_US; // MatchRegister0 setzen
		T0MR0 -= PAL_HSYNC_32_US; // MatchRegister0 setzen
	}  
	VICVectAddr = 0; // Interrupt in VIC lschen
}

//-----------------------------------------------------------------------------
// Timer0 Compare-Match Fast Interrupt Handler (ISR)
void __attribute__ ((interrupt("IRQ"))) tc0_cmp(void)
{
	PCON = 1;
	T0IR = 0xFF; // Timer1 Interrupt-Flags lschen
	VICVectAddr = 0; // Interrupt in VIC lschen
}

//-----------------------------------------------------------------------------
//EXT0 INT Compare-Match Fast Interrupt Handler (ISR)
void __attribute__ ((interrupt("IRQ"))) eint0(void) //crt0.S
{
	if (T1TC < PAL_HSYNC_32_US + 100)
	{
		pal_line_count = 0;
		pal_image_ptr = &pal_image[0];
		tmp =  (*pal_image_ptr++) << 8;
		tmp += (*pal_image_ptr++);
		T1MR0 = PAL_HSYNC_64_US + 8000;
		T0MR0 = PAL_HSYNC_64_US - 300;
		T1TC = 0;
		T0TC = 0;
		EXTINT = 1;
		VICVectAddr = 0; // Interrupt in VIC lschen
		return;
	}
	T1MR0 = PAL_HSYNC_64_US + 80000;
	T0MR0 = PAL_HSYNC_64_US - 300;
	drift = T1TC;
	T1TC = 0;
	T0TC = 0;
	
	if (++pal_line_count < LINE) //Ausgabe der Zeilen
    {	
		//Horizontaloffset
		while (T1TC < PAL_HSYNC_DURATION);// OSD_DRIFTOFFSET - drift);

		if (pal_line_count >= PAL_MIN_VERT_OFF && pal_line_count < PAL_MAX_VERT_OFF)
		{
			while(pal_bytes_to_write--)
			{
				SSPDR = tmp; // Datenbyte in den Tx-FIFO schreiben
				tmp =  (*pal_image_ptr++) << 8;
				tmp += (*pal_image_ptr++);
				while(!(SSPSR & TNF));
			}
		}
		pal_bytes_to_write = (PAL_CHAR_COLS)/2;
    }
	EXTINT = 1;
	VICVectAddr = 0; // Interrupt in VIC lschen
}
//-----------------------------------------------------------------------------

void pal_timer_init(void)
{
	pal_clear();
	PAL_SYNC_ON(); // Sync-Signal als Ausgang setzen
	
	PINSEL1 |= (1<<0); //P0.16 auf EXT0
    VICIntEnClear = 0xFFFFFFFF;
    VICIntSelect = 0x00000000;
	
	T1TCR = 0; // Timer1 ausschalten
	T1PC = 0; // Prescaler = 0
	T1EMR = 0; // Match Pin Aktivitt ausschalten
	T1CCR = 0; // Capture ausschalten
	T1PR = ((FOSC*PLL_M) / PAL_TIMER_CLK1) - 1; // Prescaler setzen
	T1MCR = TIM_MR0INT | TIM_MR0RES;// Freigabe fr Interrupt MR0 (Match-Register 0)
	T1MR0 = PAL_HSYNC_64_US; // MatchRegister0 setzen
	
	VICVectAddr0 = (long)tc1_cmp;   // set interrupt vector in 0
	VICVectCntl0 = VICVectCntl0_ENABLE | VIC_Channel_Timer1; // use it for Timer 0 Interrupt:
	VICIntEnable = (1<<VIC_Channel_Timer1);    // Enable Timer0 Interrupt
	
	T1TCR = TIM_CE; // Timer1 einschalten
	
	PCONP |= PCSPI1; // SPI1 einschalten
	PINSEL1 |= SPI1_IOSET_MASK; // PIN Konfigurieren
	SSPCPSR = 2; // Clock Prescale Register setzten
	SSPCR0 = DSS_16_BIT|FRF_SSI | (0x3<<SCR); // SSP-Format  
	SSPCR1 = SSE; // Schnittstelle als Master aktivieren
	
	
	T0TCR = 0; // Timer0 ausschalten
	T0PC = 0; // Prescaler = 0
	T0EMR = 0; // Match Pin Aktivitt ausschalten
	T0CCR = 0; // Capture ausschalten
	T0PR = ((FOSC*PLL_M) / PAL_TIMER_CLK1) - 1; // Prescaler setzen
	T0MCR = TIM_MR0INT | TIM_MR0RES;// Freigabe fr Interrupt MR0 (Match-Register 0)
	T0MR0 = PAL_HSYNC_64_US - 200; // MatchRegister0 setzen
	
	VICVectAddr2 = (long)tc0_cmp;   // set interrupt vector in 0
	VICVectCntl2 = VICVectCntl0_ENABLE | VIC_Channel_Timer0; // use it for Timer 0 Interrupt:
	VICIntEnable = (1<<VIC_Channel_Timer0);    // Enable Timer0 Interrupt
	T0TCR = TIM_CE; // Timer0 einschalten


	//Externer Interrupt
	//EXTPOLAR =(1 << 0);
	EXTMODE = (1 << 0);
	PINSEL1 |= (1<<0);
	VICVectAddr1 = (long)eint0;   	// set interrupt vector in 0
	VICVectCntl1 = (1<<5) | 14;    // use it for EXTINT0 Interrupt:
	VICIntEnable |= (1<<14);       // Enable EXTINT0 Interrupt 
	
}

//-----------------------------------------------------------------------------
void pal_encode_char(unsigned int col,unsigned int row,unsigned char chr)
{
	unsigned char *font_ptr;
	unsigned int n;
	
	if (chr >= 32)
	{
	chr -= 32;
	}
	
	font_ptr = (unsigned char *)&FONT[chr][0];
	row *= PAL_BYTES_PER_CHAR_ROW;
	
	for (n = 0; n < PAL_CHAR_LINES; n++)
	{
	pal_image[row + col + n * PAL_CHAR_COLS] = *font_ptr++;
	}
}

//-----------------------------------------------------------------------------
void pal_encode_str(unsigned int col,unsigned int row,unsigned char *pstr)
{
	while (*pstr)
	{
	pal_encode_char(col++,row,*pstr++);
	}
}

//-----------------------------------------------------------------------------
void pal_set_pixel (unsigned int zeile,unsigned int spalte)
{
	pal_image[(zeile*PAL_CHAR_COLS)+(spalte/8)]|=1 << (7- (spalte -((spalte/8)*8)));
}

//-----------------------------------------------------------------------------
void pal_clear_pixel (unsigned int zeile,unsigned int spalte)
{
	pal_image[(zeile*PAL_CHAR_COLS) + (spalte/8)]&=~(1 << (7- (spalte -((spalte/8)*8))));
}

//-----------------------------------------------------------------------------
void pal_swap_int (unsigned int *i1, unsigned int *i2 )
{
	unsigned int dummy;
	
	dummy = *i2;
	*i2 = *i1;
	*i1 = dummy;
}

//-----------------------------------------------------------------------------
//Linen zeichnen nach dem Bresenham-Algorithmus
void pal_line( unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2)
{
	int d, dx, dy, aincr, bincr, xincr, yincr, x, y;

	if (abs(x2-x1)<abs(y2-y1))
	{
		if ( y1 > y2 )
		{
			pal_swap_int( &x1, &x2 );
			pal_swap_int( &y1, &y2 );
		}
		xincr = ( x2 > x1 ) ? 1 : -1;
		dy = y2 - y1;
		dx = abs( x2-x1 );
		d = 2 * dx - dy;
		aincr = 2 * (dx - dy);
		bincr = 2 * dx;
		x = x1;
		y = y1;

		pal_set_pixel( x, y);
		for (y=y1+1; y<= y2; ++y )
		{
			if ( d >= 0 )
			{
				x += xincr;
				d += aincr;
			}
			else d += bincr;
		pal_set_pixel( x, y );
		}
	} 
	else
	{
		if ( x1 > x2 )
		{
			pal_swap_int( &x1, &x2 );
			pal_swap_int( &y1, &y2 );
		}
		yincr = ( y2 > y1 ) ? 1 : -1;
		dx = x2 - x1;
		dy = abs( y2-y1 );
		d = 2 * dy - dx;
		aincr = 2 * (dy - dx);
		bincr = 2 * dy;
		x = x1;
		y= y1;
		pal_set_pixel( x, y);
		for (x=x1+1; x<=x2; ++x )
		{
			if ( d >= 0 )
			{
				y += yincr;
				d += aincr;
			} 
			else
			{
				d += bincr;
			}
			pal_set_pixel( x, y );
		}
	}
}

//-----------------------------------------------------------------------------
void pal_clear(void)
{
	unsigned int n;
	
	for (n = 0; n < PAL_CHAR_ROWS * PAL_CHAR_LINES * PAL_CHAR_COLS; n++)
	{
		pal_image[n] = 0x00;
	}
}

