/*------------------------------------------------------------------------------
 Copyright:      Radig Ulrich mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        14.06.2007
 Description:    OSD Erzeugung
------------------------------------------------------------------------------*/

#ifndef __PAL_H
#define __PAL_H

#include "lpc213x.h"
#include "font.h"
#include "viclowlevel.h"
#include "config.h"

#define LINE 		304  // sichtbare Linien
#define VORTRABANT 	309  // 5 Vortrabanten
#define HAUPTIMPULS 314  // 5 Hauptimpulse
#define NACHTRABANT 318	 // 5 Nachtrabanten

#define PAL_MAX_LINES (unsigned int)312 // Standard PAL Zeilen pro Halbbild
#define PAL_CHAR_COLS (unsigned int)42  // immer durch 2 teilbar
#define PAL_CHAR_ROWS (unsigned int)26
#define PAL_CHAR_LINES (unsigned int)10

#define PAL_MIN_VERT_OFF (unsigned int)31
#define PAL_MAX_VERT_OFF (unsigned int)(PAL_CHAR_LINES * PAL_CHAR_ROWS + PAL_MIN_VERT_OFF)
#define PAL_TOTAL_CHAR_LINES (unsigned int)(PAL_CHAR_LINES + PAL_CHAR_BLANK_LINES)
#define PAL_BYTES_PER_CHAR_ROW (unsigned int)(PAL_CHAR_LINES * PAL_CHAR_COLS)

#define PAL_TIMER_CLK1 (unsigned int)60000000  // 1us Timer1-Takt
#define PAL_HSYNC_64_US (unsigned int)3840 // Timer Ticks fr 64.0s HSync Intervall
#define PAL_HSYNC_32_US (unsigned int)1920 // Timer Ticks fr 32.0s HSync Intervall
#define PAL_HSYNC_DURATION (unsigned int)288 // Timer Ticks fr 4.8s HSync Pulsbreite
#define PAL_HALF_HSYNC_DURATION (unsigned int)154 // Timer Ticks fr 2.4s HSync Pulsbreite	 

#define PAL_HORZ_OFF (unsigned int)500
#define OSD_PAL_HORZ_OFF (unsigned int)300

#define DRIFTOFFSET 54
#define OSD_DRIFTOFFSET 4050

#define PAL_SYNC_DIR IODIR0 // Datenrichtungsregister fr Sync-Signal
#define PAL_SYNC_SET IOSET0 // Setzregister fr Sync-Signal
#define PAL_SYNC_CLR IOCLR0 // Lschregister fr Sync-Signal
#define PAL_SYNC_PIN (1<<21) // Pin fr Sync-Signal

extern volatile unsigned char	pal_image[PAL_BYTES_PER_CHAR_ROW * PAL_CHAR_ROWS],*pal_image_ptr;
extern volatile unsigned int		pal_timer_count,pal_bytes_to_write,tmp,pal_line_count;			
extern volatile int	drift;

// Makros
#define PAL_SYNC_ON() (PAL_SYNC_DIR |= PAL_SYNC_PIN)
#define PAL_SYNC_OFF() (PAL_SYNC_DIR &= ~PAL_SYNC_PIN)

#define PAL_SYNC_HIGH() (PAL_SYNC_SET = PAL_SYNC_PIN)
#define PAL_SYNC_LOW() (PAL_SYNC_CLR = PAL_SYNC_PIN)

#define VICVectCntl0_ENABLE (1<<5)

#define VIC_Channel_Timer0  4
#define VIC_Channel_Timer1  5

#define TxTCR_COUNTER_ENABLE (1<<0)
#define TxTCR_COUNTER_RESET  (1<<1)
#define TxMCR_INT_ON_MR0     (1<<0)
#define TxMCR_RESET_ON_MR0   (1<<1)
#define TxIR_MR0_FLAG        (1<<0)
//-----------------------------------------------------------------------------

// TxIR Interrupt-Register
#define TIM_CR3INT (1<<7) // Interrupt-Flag fr Capture-Kanal 3
#define TIM_CR2INT (1<<6) // Interrupt-Flag fr Capture-Kanal 2
#define TIM_CR1INT (1<<5) // Interrupt-Flag fr Capture-Kanal 1
#define TIM_CR0INT (1<<4) // Interrupt-Flag fr Capture-Kanal 0
#define TIM_MR3INT (1<<9) // Interrupt-Flag fr Match-Kanal 3
#define TIM_MR2INT (1<<6) // Interrupt-Flag fr Match-Kanal 2
#define TIM_MR1INT (1<<3) // Interrupt-Flag fr Match-Kanal 1
#define TIM_MR0RES (1<<1)
#define TIM_MR0INT (1<<0) // Interrupt-Flag fr Match-Kanal 0

// TxTCR Timer-Control-Register
#define TIM_CR (1<<1) // Counter-Reset
#define TIM_CE (1<<0) // Counter-Enable

// TxCCR Capture-Control-Register
#define TIM_CAP3INT (1<<11) // Interrupt fr Pin CAPx.3 einschalten
#define TIM_CAP3FE  (1<<10) // Negative Flankentriggerung fr CAPx.3
#define TIM_CAP3RE  (1<<9) // Positive Flankentriggerung fr CAPx.3
#define TIM_CAP2INT (1<<8) // Interrupt fr Pin CAPx.2 einschalten
#define TIM_CAP2FE  (1<<7) // Negative Flankentriggerung fr CAPx.2
#define TIM_CAP2RE  (1<<6) // Positive Flankentriggerung fr CAPx.2
#define TIM_CAP1INT (1<<5) // Interrupt fr Pin CAPx.1 einschalten
#define TIM_CAP1FE  (1<<4) // Negative Flankentriggerung fr CAPx.1
#define TIM_CAP1RE  (1<<3) // Positive Flankentriggerung fr CAPx.1
#define TIM_CAP0INT (1<<2) // Interrupt fr Pin CAPx.0 einschalten
#define TIM_CAP0FE  (1<<1) // Negative Flankentriggerung fr CAPx.0
#define TIM_CAP0RE  (1<<0) // Positive Flankentriggerung fr CAPx.0

#define SSP_FIFO_SIZE 8

// SSPCR0 Control Register 0
#define CPHA      	(1<<7) // Clock Out Phase
#define CPOL      	(1<<6) // Clock Out Polarity
#define FRF_SPI   	0x00 // SPI MODE
#define FRF_SSI   	0x10 // SSI MODE
#define FRF_MICRO 	0x20 // Microwire MODE
#define SCR			8

#define DSS_16_BIT 0x0F

// SSPCR1 Control Register 1
#define SOD (1<<3) // Slave Output Disable
#define MS  (1<<2) // Master/Slave, 0 = MASTER
#define SSE (1<<1) // SSP Enable
#define LBM (1<<0) // Loop Back Mode

// SSPSR Status Register
#define BSY (1<<4) // BUSY
#define RFF (1<<3) // Receive FIFO Full
#define RNE (1<<2) // Receive FIFO Not Empty
#define TNF (1<<1) // Transmit FIFO Not Full
#define TFE (1<<0) // Transmit FIFO Empty

// SSPIMSC Interrupt Mask Set/Clear Register
#define TXIM  (1<<3) // IRQ Tx FIFO enable
#define RXIM  (1<<2) // IRQ Rx FIFO enable
#define RTIM  (1<<1) // IRQ Receive Timeout enable
#define RORIM (1<<0) // IRQ Receive Overrun enable

// SSPRIS Raw Interrupt Status Register
#define TXRIS  (1<<3) // Tx FIFO is at least half empty
#define RXRIS  (1<<2) // Rx FIFO is at least half full
#define RTRIS  (1<<1) // Receive Timeout condition
#define RORRIS (1<<0) // completely received while the RxFIFO was full

// SSPMIS Masked Interrupt Status Register
#define TXMIS  (1<<3) // Tx FIFO is at least half empty, and this interrupt is enabled
#define RXMIS  (1<<2) // Rx FIFO is at least half full, and this interrupt is enabled
#define RTMIS  (1<<1) // Receive Timeout condition and this interrupt is enabled
#define RORMIS (1<<0) // another frame was completely received while the RxFIFO was full, and this interrupt is enabled

// SSPICR Interrupt Clear Register
#define RTIC  (1<<1) // clears the Receive Timeout interrupt
#define RORIC (1<<0) // clears the frame was received when RxFIFO was full interrupt

// SPI-IOSET-Maske
#define SPI1_IOSET_MASK 0x000002A8 // SS,SCK,MISO und MOSI fr SPI1 einschalten

#define SPI_DI				5
#define SPI_DO				6
#define SPI_Clock			4

// SPCR Control Register 0
#define SPI0_MSTR    (1<<5) // SPI MODE Master/Slave
#define SPI0_CPOL    (1<<4) // Clock Out Polarity
#define SPI0_CPHA    (1<<3) // Clock Out Phase

// SPSR Status Register
#define SPIF    (1<<7) // BUSY

// SPI-IOSET-Maske
#define SPI0_IOSET_MASK 0x00005500 // SS,SCK,MISO und MOSI fr SPI1 einschalten

void pal_timer_init(void);
void pal_encode_char(unsigned int col,unsigned int row,unsigned char chr);
void pal_encode_str(unsigned int col,unsigned int row,unsigned char *pstr);
void pal_set_pixel (unsigned int zeile,unsigned int spalte);
void pal_clear_pixel (unsigned int zeile,unsigned int spalte);
void pal_swap_int ( unsigned int *i1, unsigned int *i2 );
void pal_line( unsigned int x1, unsigned int y1, unsigned int x2, unsigned int y2);
void pal_clear (void);

//-----------------------------------------------------------------------------

#endif
